/*
   Copyright 2007-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.clip;

import org.apache.log4j.Logger;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * A utility to do a one-time grab of clipboard contents when it matches a given
 * pattern.
 *
 * User: murali
 * Date: Sep 2, 2008
 * Time: 12:25:39 PM
 */
public class ClipboardGrab implements ClipboardListener {

    //------------------------------------------------------------
    //- Class Variables

    private static final Logger LOG = Logger.getLogger("com.notuvy.clip");

    //------------------------------------------------------------
    //- Class Functions

    //------------------------------------------------------------
    //- Instance Variables

    private final Pattern fPattern;
    private boolean fSaved = false;
    private String fContent = "";

    //------------------------------------------------------------
    //- Constructors

    public ClipboardGrab(Pattern pPattern) {
        fPattern = pPattern;
    }

    //------------------------------------------------------------
    //- Getters

    protected Pattern getPattern() {
        return fPattern;
    }

    //------------------------------------------------------------
    //- Settors

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected void grab() {
        ClipboardCallback.register(this);
        LOG.info("Waiting for content in the clipboard.");

        while (!fSaved) {
            try {
                Thread.sleep(250);
            } catch (InterruptedException e) {
                LOG.warn("Thread interrupted.");
            }
        }
        ClipboardCallback.deregister(this);
    }

    //------------------------------------------------------------
    //- Public Interface Functions

    public String getContent() {
        if (!fSaved) {
            grab();
        }

        return fContent;
    }

    //------------------------------------------------------------
    //- Class Interface Functions

    public void newClipboardContents(String pString) {
        Matcher matcher = getPattern().matcher(pString);
        if (matcher.find()) {
            fContent = pString;
            fSaved = true;
        }
    }

    //------------------------------------------------------------
    //- Inner Classes

    //------------------------------------------------------------
    //- Main

}
