/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.cmd;

import org.apache.log4j.Logger;

import java.util.*;
import java.util.regex.Pattern;

/**
 * Process all arguments, and using given regex's, split them into command arguments
 * (switchs) and program arguments (application data).
 *
 * User: murali
 * Date: Jan 25, 2008
 * Time: 9:22:02 AM
 */
public class AllArgs {

    //------------------------------------------------------------
    //- Class Variables

    private static final Logger gLogger = Logger.getLogger("com.notuvy.cmd");

    //------------------------------------------------------------
    //- Class Functions

    //------------------------------------------------------------
    //- Instance Variables

    private final List<String> fArgsCommand = new ArrayList<String>();
    private final List<String> fArgsProgram = new ArrayList<String>();
    private final List<Pattern> fPatternsCommand = new ArrayList<Pattern>();

    //------------------------------------------------------------
    //- Constructors

    //------------------------------------------------------------
    //- Getters

    public List<String> getArgsCommand() {
        return fArgsCommand;
    }

    public List<String> getArgsProgram() {
        return fArgsProgram;
    }

    private List<Pattern> getPatternsCommand() {
        return fPatternsCommand;
    }

    //------------------------------------------------------------
    //- Settors

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    //------------------------------------------------------------
    //- Public Interface Functions

    public CommandArgs toCommandArgs() {
        return new CommandArgs(getArgsCommand());
    }

    public AllArgs addPattern(String pString) {
        getPatternsCommand().add(Pattern.compile(pString));
        return this;
    }

    public AllArgs addDefined(CommandSet pSet) {
        for (String key : pSet.getMap().keySet()) {
            getPatternsCommand().add(Pattern.compile("^"+key+"$"));
        }
        return this;
    }

    public AllArgs process(String[] pArgs) {
        List<String> remaining = new ArrayList<String>(Arrays.asList(pArgs));
        for (String arg : pArgs) {
            boolean thisPatternMatched = false;
            Iterator<Pattern> patterns = getPatternsCommand().iterator();
            while (!thisPatternMatched && patterns.hasNext()) {
                Pattern pattern = patterns.next();
                if (pattern.matcher(arg).find()) {
                    thisPatternMatched = true;
                    getArgsCommand().add(arg);
                    remaining.remove(arg);
                }
            }
        }
        getArgsProgram().addAll(remaining);
        return this;
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    public String toString() {
        return "Args[Command" + getArgsCommand() + ", Program" + getArgsProgram() + "]";
    }

    //------------------------------------------------------------
    //- Inner Classes

    //------------------------------------------------------------
    //- Main

}
