/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.cmd;

import org.apache.commons.lang.StringUtils;

import java.util.List;
import java.util.ArrayList;

/**
 * A single command performing an actual action.  It is subclassed with an implementation of
 * the process() method.
 *
 * User: murali
 * Date: Jan 2, 2008
 * Time: 2:10:35 PM
 */
public abstract class Command extends CommandAction {

    //------------------------------------------------------------
    //- Class Variables


    //------------------------------------------------------------
    //- Class Functions

    //------------------------------------------------------------
    //- Instance Variables

    private final String fArgumentsString;
    private final String fBody;


    //------------------------------------------------------------
    //- Constructors

    protected Command(String pKey, String pArgumentsString) {
        super(pKey);

        fArgumentsString = pArgumentsString;
        fBody = (StringUtils.isBlank(pArgumentsString)) ?
                pKey :
                pKey + " " + pArgumentsString;
    }

    protected Command(String pKey) {
        this (pKey, null);
    }

    //------------------------------------------------------------
    //- Getters

    public String getArgumentsString() {
        return fArgumentsString;
    }

    protected String getBody() {
        return fBody;
    }

    //------------------------------------------------------------
    //- Settors

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    //------------------------------------------------------------
    //- Public Interface Functions

    //------------------------------------------------------------
    //- Class Interface Functions

    public List<String> listCommands() {
        List<String> result = new ArrayList<String>(1);
        result.add(getBody());

        return result;
    }


    /**
     * The principle logic of this command.
     * It must return true if successfully executed, or false if there was a problem and processing must be aborted.
     * The CommandArgs must be properly managed.  It must assert that all the required arguments are present.
     *
     * @param pArgs  The argument available list.  Note that it will contain the arguments for this command
     *               plus all follow-on commands and their arguments.
     * @return  The status flag; true for success (processing proceeds) and false for failure (processing aborts).
     */
    public abstract boolean process(CommandArgs pArgs);


    protected boolean handle(CommandArgs pArgs) {
        boolean valid = process(pArgs);
        if (!valid) {
            LOG.error(toUsageString());
        }
        return valid;
    }

    public String toUsageString() {
        String parents = (getParent() == null) ? "" : getParent().constructParentCommands();
        return "Usage: " + parents + " " + getBody();
    }

    //------------------------------------------------------------
    //- Inner Classes

    //------------------------------------------------------------
    //- Main

}
