/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.cmd;

import org.apache.log4j.Logger;
import org.apache.commons.lang.StringUtils;

import java.util.Queue;
import java.util.List;

/**
 * The generic definition of a command action.  A command (or command subset) needs to be
 * able to list its name (key), list its usage, and to process arguments.
 *  
 * User: murali
 * Date: Jan 2, 2008
 * Time: 2:09:44 PM
 */
public abstract class CommandAction {

    protected static final Logger LOG = Logger.getLogger("com.notuvy.cmd");


    private final String fKey;
    private CommandAction fParent = null;
    private boolean fTerminated = false;

    CommandAction(String pKey) {
        fKey = pKey;
    }

    protected String getKey() {
        return fKey;
    }

    protected CommandAction getParent() {
        return fParent;
    }

    public boolean isTerminated() {
        return fTerminated;
    }

    protected void setParent(CommandAction pParent) {
        fParent = pParent;
    }

    public void setTerminated(boolean pTerminated) {
        fTerminated = pTerminated;
    }

    protected abstract List<String> listCommands();

    protected abstract boolean handle(CommandArgs pArgs);

    protected abstract String toUsageString();

    protected String constructParentCommands() {
        String result = "";
        if (getParent() != null) {
            String parentCommand = getParent().constructParentCommands();
            result = StringUtils.isBlank(parentCommand) ? getKey() : parentCommand + " " + getKey();
        }
        return result;
    }

    public boolean process(String[] pStringArgs) {
        return process(new CommandArgs(pStringArgs));
    }

    public boolean process(CommandArgs pArgs) {
        return handle(pArgs);
    }
}
