/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.cmd;

import org.apache.log4j.Logger;

import java.util.LinkedList;
import java.util.List;

/**
 * The list of arguments to be processed.
 *
 * User: murali
 * Date: Jan 4, 2008
 * Time: 10:40:48 AM
 */
public class CommandArgs {

    //------------------------------------------------------------
    //- Class Variables

    private static final Logger LOG = Logger.getLogger("com.notuvy.cmd");

    //------------------------------------------------------------
    //- Class Functions

    //------------------------------------------------------------
    //- Instance Variables

    private final LinkedList<String> fValues = new LinkedList<String>();


    //------------------------------------------------------------
    //- Constructors

    protected CommandArgs(String[] pArgs) {
        for (String arg : pArgs) {
            getValues().add(arg);
        }
    }

    protected CommandArgs(List<String> pList) {
        getValues().addAll(pList);
    }

    //------------------------------------------------------------
    //- Getters

    private LinkedList<String> getValues() {
        return fValues;
    }

    //------------------------------------------------------------
    //- Settors

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected boolean isEmpty() {
        return getValues().isEmpty();
    }

    //------------------------------------------------------------
    //- Public Interface Functions

    public boolean hasNext() {
        return !getValues().isEmpty();
    }

    /**
     * This should be used in conjunction with "hasNext()" since this does no error checking.
     * @return  The first item currently in the list.
     */
    public String peek() {
        return getValues().getFirst();
    }

    public String next() {
        return getValues().remove();
    }

    public boolean exhausted() {
        boolean empty = getValues().isEmpty();
        if (!empty) {
            LOG.error("unrecognized extra " + getValues().size() + " arguments: " + getValues());
        }
        return empty;
    }

    public boolean hasSwitch(String pFlag) {
        boolean result = false;  // Fallback.

        if (getValues().contains(pFlag)) {
            result = true;
            //noinspection StatementWithEmptyBody
            while (getValues().remove(pFlag)) ;  // Remove all.
        }
        return result;
    }

    public String switchValue(String pFlag) {
        String result = "";  // Fallback.

        if (getValues().contains(pFlag)) {
            // It is not valid if there is no value given after it.  This is when
            // the switch is the last, or it is the only.
            if (getValues().getLast().equals(pFlag) || (getValues().size() == 1)) {
                LOG.error("No value given for switch [" + pFlag + "]");
            } else {
                int index = getValues().indexOf(pFlag);
                getValues().remove(index);  // first the switch
                result = getValues().remove(index);  // then the value
            }
        }
        return result;
    }

    //------------------------------------------------------------
    //- Class Interface Functions

    public String toString() {
        return getValues().toString();
    }

    //------------------------------------------------------------
    //- Inner Classes

    //------------------------------------------------------------
    //- Main

}
