package com.notuvy.condtrig;

import org.apache.commons.lang.StringUtils;

/**
 * A single boolean value.
 * These is the part of the expression than can be explicitly manipulated (via the public set() method).
 * An change in value will ripple up through ancesctor expression and cause the whole antecedant to be recalculated.
 *
 * @author murali
 */
public class Value extends Expression {

    //------------------------------------------------------------
    //- Class Variables


    //------------------------------------------------------------
    //- Class Functions

    public static Value of(boolean pInitial) {
        return of(pInitial, "");
    }

    public static Value of(boolean pInitial, String pName) {
        Value result = new Value(pInitial);
        result.fName = pName;
        return result;
    }


    //------------------------------------------------------------
    //- Instance Variables

    private String fName = "";


    //------------------------------------------------------------
    //- Constructors

    private Value(boolean pValue) {
        super (pValue);
    }


    //------------------------------------------------------------
    //- Gettors

    //------------------------------------------------------------
    //- Settors

    public void set(boolean pNewValue) {
        // Because setValue(boolean) in the parent is not publicly accessible
        // (and it should not be), we need a separate public method to make
        // leaf values settable.
        setValue(pNewValue);
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    public boolean isTrue() {
        return getValue();
    }

    public boolean isFalse() {
        return !isTrue();
    }

    public void setTrue() {
        setValue(true);
    }

    public void setFalse() {
        setValue(false);
    }

    public void toggle() {
        setValue(!getValue());
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    protected void recalculate() {
        // Nothing
    }

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder();
        builder.append("(");
        if (StringUtils.isNotBlank(fName)) {
            builder.append(fName).append(":");
        }
        builder.append(getValue());
        builder.append(")");
        return builder.toString();
    }


    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main


}
