/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.file;

import org.apache.log4j.Logger;

import java.io.File;
import java.util.List;
import java.util.ArrayList;

/**
 * The contents of a given parent directory, split between plain files
 * and sub-directories.
 *
 * User: murali
 * Date: Jan 27, 2008
 * Time: 3:07:45 PM
 */
public class DirectoryContents {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger("com.notuvy.file");

    //------------------------------------------------------------
    //- Class Functions

    //------------------------------------------------------------
    //- Instance Variables

    private final Directory fParent;
    private final List<File> fPlainFiles = new ArrayList<File>();
    private final List<Directory> fSubDirectories = new ArrayList<Directory>();

    //------------------------------------------------------------
    //- Constructors

    public DirectoryContents(Directory pParent) {
        fParent = pParent;
        for (File file : pParent.listContents()) {
            if (file.isDirectory()) {
                getSubDirectories().add(new Directory(file));
            } else {
                getPlainFiles().add(file);
            }
        }
    }

    //------------------------------------------------------------
    //- Accessors

    public Directory getParent() {
        return fParent;
    }

    public List<File> getPlainFiles() {
        return fPlainFiles;
    }

    public List<Directory> getSubDirectories() {
        return fSubDirectories;
    }

    //------------------------------------------------------------
    //- Settors

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    //------------------------------------------------------------
    //- Public Interface Functions

    //------------------------------------------------------------
    //- Class Interface Functions

    //------------------------------------------------------------
    //- Inner Classes

    //------------------------------------------------------------
    //- Main

}
