/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.file;

import org.apache.log4j.Logger;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

/**
 * Report what type of EOL (Windows or unix) is used in the given file.
 *
 * User: murali
 * Date: Sep 19, 2008
 * Time: 12:54:59 PM
 */
public class EolCheck {

    private static final byte MAX_ASCII_VALUE = 127;
    private static final byte LF = 10;
    private static final byte CR = 13;

    //------------------------------------------------------------
    //- Class Variables

    private static final Logger LOG = Logger.getLogger(EolCheck.class);

    //------------------------------------------------------------
    //- Class Functions

    //------------------------------------------------------------
    //- Instance Variables

    //------------------------------------------------------------
    //- Constructors

    //------------------------------------------------------------
    //- Getters

    //------------------------------------------------------------
    //- Settors

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    //------------------------------------------------------------
    //- Public Interface Functions

    public String check(String pFilename) {
        String result = "unknown";
        File file = new File(pFilename);
        if (file.exists()) {
            if (file.isDirectory()) {
                result = "directory";
            } else {
                boolean textFile = true;  // Optimistic
                boolean crlf = false;
                try {
                    byte[] chunk = new byte[1024];

                    byte last = -1;
                    FileInputStream fis = new FileInputStream(file);
                    for (int numRead=fis.read(chunk); (numRead!=-1)&&textFile; numRead=fis.read(chunk)) {
                        for (int i=0; i<numRead; i++) {
                            byte current = chunk[i];
                            textFile = (current <= MAX_ASCII_VALUE);
                            if (textFile) {
                                if ((current == LF) && (last == CR)) {
                                    crlf = true;
                                }
                            }
                            last = current;
                        }
                    }
                    fis.close();

                    if (textFile) {
                        result = crlf ? "Windows EOL" : "Unix EOL";
                    } else {
                        result = "not a text file";
                    }
                } catch (IOException ioe) {
                    LOG.error("Problem reading file.", ioe);
                }
            }
        } else {
            result = "no such file";
        }
        return result;
    }

    public void report(String pFilename) {
        System.out.println(pFilename + ": " + check(pFilename));
    }

    //------------------------------------------------------------
    //- Class Interface Functions

    //------------------------------------------------------------
    //- Inner Classes

    //------------------------------------------------------------
    //- Main

    public static void main(String[] args) {
        EolCheck checker = new EolCheck();
        for (String arg : args) {
            checker.report(arg);
        }
    }

}
