/*
   Copyright 2004-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.file;

import java.io.*;
import java.util.*;

import org.apache.log4j.Logger;

/**
 * A programatic file file.  Deep dir search.
 * 
 * @author  murali
 */
public class FileFind implements Runnable {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(FileFind.class);

    public static final FileFilter F_PLAIN = new FileFilter() {
            public boolean accept(File file) {
                boolean result = file.isFile();
                return result;
            }
        };


    public static final FileFilter F_DIRS = new FileFilter() {
            public boolean accept(File file) {
                boolean result = file.isDirectory() && !"CVS".equals(file.getName());
                return result;
            }
        };


    //------------------------------------------------------------
    //- Class Functions



    //------------------------------------------------------------
    //- Instance Variables

    private File vParent = null;
    private final FileFilter vFilter;
    private final List vFiles = new ArrayList();
    private boolean vInitialized = false;


    //------------------------------------------------------------
    //- Constructors

    public FileFind(File pParent, FileFilter pFilter) {
        vParent = pParent;
        vFilter = pFilter;

        run();
    }

    public FileFind(File pParent) {
        this (pParent, F_PLAIN);
    }


    public FileFind(String filename, FileFilter pFilter) {
        this (new File(filename), pFilter);
    }


    public FileFind(String filename) {
        this (new File(filename), F_PLAIN);
    }


    //------------------------------------------------------------
    //- Accessors

    public File getParent() {
        return vParent;
    }

    protected FileFilter getFilter() {
        return vFilter;
    }

    protected List getFiles() {
        return vFiles;
    }

    protected boolean isInitialized() {
        return vInitialized;
    }


    //------------------------------------------------------------
    //- Settors

    protected void setInitialized(boolean pInitialized) {
        vInitialized = pInitialized;
    }



    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    public synchronized void run() {
        if (!isInitialized()) {
            if (getParent().isDirectory()) {
                findIn(getParent());
                Collections.sort(getFiles());
            }
            setInitialized(true);
        }
    }


    protected void findIn(File dir) {
        File[] contents = dir.listFiles(getFilter());
        for (int i = 0; i < contents.length; i++) {
            getFiles().add(contents[i]);
        }

        File[] subdirs = dir.listFiles(F_DIRS);
        for (int i = 0; i < subdirs.length; i++) {
            findIn(subdirs[i]);
        }
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public String toString() {
        StringBuffer buffer = new StringBuffer("FileFind[" + getParent() + "][" + listFiles().size() + "]:\n");

        Iterator iterator = listFiles().iterator();
        while (iterator.hasNext()) {
            File file = (File) iterator.next();
            buffer.append("  ").append(file.getAbsolutePath()).append("\n");
        }

        return buffer.toString();
    }


    public Collection listFiles() {
        run();

        Collection result = getFiles();
        return result;
    }


    //------------------------------------------------------------
    //- Class Interface Functions



    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main

    public static void main(String[] args) {
        try {
            for (int i = 0; i < args.length; i++) {
                FileFind find = new FileFind(args[i]);
                LOG.info(find.listFiles());
            }
        } catch (Throwable thr) {
            LOG.error("Fatal error.", thr);
        }
    }


}
