/*
   Copyright 2007-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.gui;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import org.apache.log4j.Logger;

/**
 * A UI component that includes a uneditable text field with an attached
 * button that can be used to launch and editor that controls the text.
 *
 * @author  murali
 */
public class ButtonField extends JPanel implements FormComponent {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(ButtonField.class);


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final JTextField vField = new JTextField();
    private final JButton vButton = new JButton();


    //------------------------------------------------------------
    //- Constructors

    public ButtonField(String pFieldText, String pButtonText) {
        super (new BorderLayout());

        getField().setText(pFieldText);
        getField().setEditable(false);

        getButton().setMargin(new Insets(0, 0, 0, 0));
        getButton().setText(pButtonText);

        add(getField(), BorderLayout.CENTER);
        add(getButton(), BorderLayout.EAST);
    }


    //------------------------------------------------------------
    //- Accessors

    public JTextField getField() {
        return vField;
    }

    public JButton getButton() {
        return vButton;
    }


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions



    //------------------------------------------------------------
    //- Class Interface Functions

    public JComponent toComponent() {
        return this;
    }


    public Object getValue() {
        String result = getField().getText();
        return result;
    }

    public void setValue(Object object) {
        getField().setText(String.valueOf(object));
    }


    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main



}
