/*
   Copyright 2007-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.gui;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import org.apache.log4j.Logger;

import com.notuvy.util.IconResource;

/**
 * A UI component for a filename.  It is a panel that includes a text field
 * for the file name, and a button to launch a file chooser.
 *
 * @author  murali
 */
public abstract class FilenameField extends JPanel implements FormComponent, ActionListener {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(FilenameField.class);

    private static final Icon OPEN_ICON = IconResource.get("/file_open.gif");


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final JTextField vFieldFilename = new JTextField(40);
    private final JButton vButtonChooser = new JButton(OPEN_ICON);
    private final int vSelctionMode;


    //------------------------------------------------------------
    //- Constructors

    protected FilenameField(int pSelctionMode) {
        super (new BorderLayout());

        vSelctionMode = pSelctionMode;

        getButtonChooser().setMargin(new Insets(0, 0, 0, 0));
        getButtonChooser().setToolTipText("Choose file");

        getButtonChooser().addActionListener(this);
        getFieldFilename().addActionListener(this);

        add(getFieldFilename(), BorderLayout.CENTER);
        add(getButtonChooser(), BorderLayout.EAST);
    }


    //------------------------------------------------------------
    //- Accessors

    public JTextField getFieldFilename() {
        return vFieldFilename;
    }

    public JButton getButtonChooser() {
        return vButtonChooser;
    }

    protected int getSelctionMode() {
        return vSelctionMode;
    }


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions



    //------------------------------------------------------------
    //- Class Interface Functions

    public JComponent toComponent() {
        return this;
    }


    public Object getValue() {
        String result = getFieldFilename().getText();
        return result;
    }

    public void setValue(Object object) {
        getFieldFilename().setText(String.valueOf(object));
    }


    public void actionPerformed(ActionEvent ae) {
        JFileChooser chooser = new JFileChooser(getFieldFilename().getText());

        chooser.setFileSelectionMode(getSelctionMode());

        int response = chooser.showOpenDialog(null);
        if (response == JFileChooser.APPROVE_OPTION) {
            getFieldFilename().setText(chooser.getSelectedFile().getAbsolutePath());
        }
    }


    //------------------------------------------------------------
    //- Inner Classes

    public static class Dirs extends FilenameField {
        public Dirs() {
            super (JFileChooser.DIRECTORIES_ONLY);
        }
    }


    public static class Files extends FilenameField {
        public Files() {
            super (JFileChooser.FILES_ONLY);
        }
    }


    public static class Both extends FilenameField {
        public Both() {
            super (JFileChooser.FILES_AND_DIRECTORIES);
        }
    }


    //------------------------------------------------------------
    //- Main



}
