package com.notuvy.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;


/**
 * Display a non-modal window displaying static text.  The only operation on it is to dismiss it.
 *
 * User: murali
 * Date: Nov 1, 2010
 * Time: 4:10:23 PM
 */
public class FloatingInfoWindow extends GenericFrame {

    //------------------------------------------------------------
    //- Class Variables

    private static final Font DEFAULT_FONT = new Font("Monospaced", Font.PLAIN, 14);
    private static final Font TITLE_LABEL_FONT = new Font("Verdana", Font.BOLD, 14);


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final String fReference;
    private final JTextArea fArea;
    private final JButton fClose = new JButton("OK");

    //------------------------------------------------------------
    //- Constructors

    public FloatingInfoWindow(String pTitle, String pContent, WindowLocation pWindowLocation) {
        super (pWindowLocation);

        fReference = pTitle.replaceAll("\\s+", "").toLowerCase();
        getFrame().setTitle(pTitle);
        fArea = new JTextArea(pContent);
    }

    //------------------------------------------------------------
    //- Getters

    public String getReference() {
        return fReference;
    }

    public JTextArea getArea() {
        return fArea;
    }

    protected JButton getClose() {
        return fClose;
    }


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected void initializeUIComponents() {
        getClose().addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent pActionEvent) {
                handleClose();
            }
        });

        getArea().setEditable(false);
        getArea().setFont(DEFAULT_FONT);
    }

    protected void buildUI(Container pContainer) {
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout(FlowLayout.CENTER, 4, 4));
        buttonPanel.add(getClose());

        JLabel label = new JLabel(getFrame().getTitle());
        label.setFont(TITLE_LABEL_FONT);
        JPanel labelPanel = new JPanel();
        labelPanel.setLayout(new FlowLayout(FlowLayout.CENTER, 4, 4));
        labelPanel.add(label);

        pContainer.setLayout(new BorderLayout());
        pContainer.add(labelPanel, BorderLayout.NORTH);
        pContainer.add(getArea(), BorderLayout.CENTER);
        pContainer.add(buttonPanel, BorderLayout.SOUTH);
    }


    //------------------------------------------------------------
    //- UI Action Handling


    //------------------------------------------------------------
    //- Public Interface Functions


    //------------------------------------------------------------
    //- Class Interface Functions

    @Override
    public void launch() {
        super.launch();

        getClose().requestFocus();
    }

    public String giveWindowName() {
        return getReference();
    }

    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main

}
