/*
   Copyright 2007-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.gui;

import java.awt.Container;
import java.awt.Window;

import javax.swing.*;

/**
 * An abstract framework for creating a GUI frame for an application.
 *
 * @author  murali
 */
public abstract class GenericDialog extends GenericWindow {

    //------------------------------------------------------------
    //- Class Variables



    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final JDialog fDialog;


    //------------------------------------------------------------
    //- Constructors

    protected GenericDialog(WindowLocation pWindowLocation) {
        super(pWindowLocation);
        fDialog = new JDialog();
        getDialog().setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
    }

    protected GenericDialog(WindowLocation pWindowLocation, JFrame pOwner) {
        super(pWindowLocation);
        fDialog = new JDialog(pOwner);
        getDialog().setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
    }

    protected GenericDialog(WindowLocation pWindowLocation, JDialog pOwner) {
        super(pWindowLocation);
        fDialog = new JDialog(pOwner);
        getDialog().setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
    }


    //------------------------------------------------------------
    //- Accessors

    protected JDialog getDialog() {
        return fDialog;
    }


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    public void handleClose() {
        getWindowLocation().remember(this);
        getDialog().dispose();
    }


    //------------------------------------------------------------
    //- Public Interface Functions


    //------------------------------------------------------------
    //- Class Interface Functions

    public Window giveLocatableWindow() {
        return getDialog();
    }

    protected Container getContentPane() {
        return getDialog().getContentPane();
    }


    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main



}