/*
   Copyright 2007-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.gui;

import java.awt.Container;
import java.awt.Window;

import javax.swing.*;

/**
 * An abstract framework for creating a GUI frame for an application.
 *
 * @author  murali
 */
public abstract class GenericFrame extends GenericWindow {

    //------------------------------------------------------------
    //- Class Variables



    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final JFrame vFrame = new JFrame();


    //------------------------------------------------------------
    //- Constructors

    protected GenericFrame(WindowLocation pWindowLocation) {
        super(pWindowLocation);

        getFrame().setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
    }


    //------------------------------------------------------------
    //- Accessors

    protected JFrame getFrame() {
        return vFrame;
    }


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    public void handleClose() {
        getWindowLocation().remember(this);
        getFrame().setVisible(false);
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    /**
     * It may be necessary to programmatically deiconify and bring
     * the window to the front.  Handle that here, and make it public
     * for general use.
     */
    public void raise() {
        getFrame().setState(JFrame.NORMAL);
        getFrame().setVisible(true);
        getFrame().toFront();
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    public Window giveLocatableWindow() {
        return getFrame();
    }

    protected Container getContentPane() {
        return getFrame().getContentPane();
    }


    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main



}
