package com.notuvy.gui;

import javax.swing.*;
import java.util.LinkedList;
import java.awt.*;

/**
 * Basic description
 *
 * @author murali
 */
public class GenericMenuBuilder {

    //------------------------------------------------------------
    //- Class Variables



    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final LinkedList<ItemWrapper> fContents = new LinkedList<ItemWrapper>();


    //------------------------------------------------------------
    //- Constructors


    //------------------------------------------------------------
    //- Gettors


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    public GenericMenuBuilder addSeparator() {
        if (!fContents.isEmpty()) {
            fContents.getLast().postSeparated = true;
        }
        return this;
    }

    public GenericMenuBuilder add(JMenuItem pItem) {
        return add(pItem, -1);
    }

    public GenericMenuBuilder add(JMenuItem pItem, int pMnemonic) {
        ItemWrapper wrapper = new ItemWrapper(pItem);
        if (pMnemonic > 0) {
            pItem.setMnemonic(pMnemonic);
        }
        fContents.add(wrapper);
        return this;
    }

    public GenericMenuBuilder add(MenuItem pItem) {
        ItemWrapper wrapper = new ItemWrapper(pItem);
        fContents.add(wrapper);
        return this;
    }

    public JMenu toSubMenu(String pName) {
        return toSubMenu(pName, -1);
    }

    public JMenu toSubMenu(String pName, int pMnemonic) {
        JMenu result = new JMenu(pName);
        for (ItemWrapper wrapper : fContents) {
            result.add(wrapper.item);
            if (wrapper.postSeparated) {
                result.addSeparator();
            }
        }
        if (pMnemonic > 0) {
            result.setMnemonic(pMnemonic);
        }
        return result;
    }

    public JPopupMenu toPopup() {
        JPopupMenu result = new JPopupMenu();
        fill(result);
        return result;
    }

    public void fill(JPopupMenu pPopupMenu) {
        for (ItemWrapper wrapper : fContents) {
            pPopupMenu.add(wrapper.item);
            if (wrapper.postSeparated) {
                pPopupMenu.addSeparator();
            }
        }
    }

    public void fill(PopupMenu pPopupMenu) {
        for (ItemWrapper wrapper : fContents) {
            pPopupMenu.add(wrapper.itemAWT);
            if (wrapper.postSeparated) {
                pPopupMenu.addSeparator();
            }
        }
    }


    //------------------------------------------------------------
    //- Class Interface Functions


    //------------------------------------------------------------
    //- Inner Classes

    private static class ItemWrapper {
        private boolean postSeparated = false;
        private final JMenuItem item;
        private final MenuItem itemAWT;

        private ItemWrapper(JMenuItem pItem) {
            item = pItem;
            itemAWT = null;
        }

        private ItemWrapper(MenuItem pItem) {
            item = null;
            itemAWT = pItem;
        }
    }

    //------------------------------------------------------------
    //- Main


}
