package com.notuvy.gui;


import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/**
 * An abstract framework for creating a GUI window (either frame or dialog) for an application.
 *
 * User: murali
 * Date: Nov 15, 2010
 * Time: 12:36:40 PM
 */
public abstract class GenericWindow implements PersistentLocatable {

    //------------------------------------------------------------
    //- Class Variables



    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private boolean fInitialized = false;
    protected final WindowLocation fWindowLocation;


    //------------------------------------------------------------
    //- Constructors

    public GenericWindow(WindowLocation pWindowLocation) {
        fWindowLocation = pWindowLocation;
    }


    //------------------------------------------------------------
    //- Getters

    protected boolean isInitialized() {
        return fInitialized;
    }

    protected WindowLocation getWindowLocation() {
        return fWindowLocation;
    }


    //------------------------------------------------------------
    //- Settors

    protected void setInitialized(boolean pInitialized) {
        fInitialized = pInitialized;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    public void launch() {
        if (isInitialized()) {
            giveLocatableWindow().setVisible(true);
        } else {
            initializeUIComponents();
            buildUI(getContentPane());

            getWindowLocation().set(this);
            giveLocatableWindow().pack();
            giveLocatableWindow().setVisible(true);

            giveLocatableWindow().addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    attemptClose();
                }
            });

            setInitialized(true);
        }
    }

    public void attemptClose() {
        if (allowWindowClose()) {
            handleClose();
        }
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    /**
     * Subclasses can override this method to intercept the window closing event and
     * check whether the window close should actually be allowed.
     *
     * @return false if the window close should be rejected, true to let it close.
     */
    protected boolean allowWindowClose() {
        return true;
    }

    public abstract void handleClose();

    protected abstract void initializeUIComponents();

    protected abstract void buildUI(Container pContainer);

    protected abstract Container getContentPane();


    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main

}
