/*
   Copyright 2007-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.gui;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import com.notuvy.util.IconResource;

/**
 * Basic description
 *
 * @author  murali
 */
public class InfoButton extends JButton implements ActionListener {

    //------------------------------------------------------------
    //- Class Variables

    private static final Icon INFO_ICON = IconResource.get("/info_icon.gif");
    private static final Insets NONE = new Insets(0, 0, 0, 0);


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final Component vOwner;
    private String vTitle = "";
    private String vBodyText = "";


    //------------------------------------------------------------
    //- Constructors

    public InfoButton(final Component pOwner, final String pTitle, final String pBodyText) {
        super (INFO_ICON);

        vOwner = pOwner;
        setTitle(pTitle);
        setBodyText(pBodyText);

        setBorderPainted(false);
        setMargin(NONE);
        setToolTipText("Click for description.");

        addActionListener(this);
    }


    //------------------------------------------------------------
    //- Accessors

    protected Component getOwner() {
        return vOwner;
    }

    public String getTitle() {
        return vTitle;
    }

    public String getBodyText() {
        return vBodyText;
    }


    //------------------------------------------------------------
    //- Settors

    public void setTitle(String pTitle) {
        vTitle = pTitle;
    }

    public void setBodyText(String pBodyText) {
        vBodyText = pBodyText;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    public void actionPerformed(ActionEvent ae) {
        JOptionPane.showMessageDialog(getOwner(), getBodyText(), getTitle(), JOptionPane.INFORMATION_MESSAGE);
    }


    //------------------------------------------------------------
    //- Class Interface Functions



    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main



}
