/*
   Copyright 2007-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.gui;

import java.awt.*;
import javax.swing.*;

import org.apache.log4j.Logger;

import com.notuvy.util.BooleanStateHandler;
import com.notuvy.util.IconResource;

/**
 * A label with an icon that looks like an LED that displays state.
 *
 * @author  murali
 */
public class LEDLabel extends JLabel implements BooleanStateHandler {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(LEDLabel.class);

    public static final Icon RED = IconResource.get("/led_red.gif");
    public static final Icon GREEN = IconResource.get("/led_green.gif");


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private boolean vState = false;


    //------------------------------------------------------------
    //- Constructors

    public LEDLabel(String pText) {
        super (pText, RED, TRAILING);
    }


    //------------------------------------------------------------
    //- Accessors

    public boolean getState() {
        return vState;
    }


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    public void setState(boolean pState) {
        vState = pState;

        setIcon(pState ? GREEN : RED);
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public void toggle() {
        setState(!getState());
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    public void initializeState(boolean pValue) {
        setState(pValue); 
    }

    public void handleState(boolean pValue) {
        setState(pValue); 
    }


    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main



}
