package com.notuvy.gui;

import org.apache.log4j.Logger;
import com.notuvy.util.WallClock;
import com.notuvy.thread.ContinuousExecution;
import com.notuvy.thread.IdlenessMonitorable;

import java.awt.*;

/**
 * Basic description
 *
 * @author murali
 */
public class MouseActivityMonitor implements Runnable, IdlenessMonitorable {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger("com.notuvy.gui");

    private static final MouseActivityMonitor INSTANCE = new MouseActivityMonitor();

    // Create a separate thread that fires every one second to check if the mouse has moved.
    private static final ContinuousExecution RUNNER = new ContinuousExecution(1000);

    static {
        RUNNER.add(new Runner());
        RUNNER.startThread("MouseActivityMonitor", true);
    }


    //------------------------------------------------------------
    //- Class Functions

    public static boolean hasMouseMovedSinceLast(long pMillisPeriod) {
        return INSTANCE.fActivityMonitor.getLastUIActivity() >= (WallClock.currentTimeMillis() - pMillisPeriod);
    }

    public static MouseActivityMonitor getInstance() {
        return INSTANCE;
    }


    //------------------------------------------------------------
    //- Instance Variables

    private final UIActivityMonitor fActivityMonitor = new UIActivityMonitor();
    private Point fLastLocation = new Point(0, 0);


    //------------------------------------------------------------
    //- Constructors

    private MouseActivityMonitor() {
        // private for singleton.
    }


    //------------------------------------------------------------
    //- Gettors


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    private void checkMouse() {
        Point currentLocation = MouseInfo.getPointerInfo().getLocation();
        if (!currentLocation.equals(fLastLocation)) {
            fActivityMonitor.touchUIActivity();
            fLastLocation = currentLocation;
        }
    }


    //------------------------------------------------------------
    //- Public Interface Functions


    //------------------------------------------------------------
    //- Class Interface Functions

    @Override
    public String toString() {
        return String.format("Mouse at [%s] since [%s].", fLastLocation, fActivityMonitor);
    }

    public long lastActivity() {
        return fActivityMonitor.lastActivity();
    }

    public void markActionTakenOnIdle() {
        fActivityMonitor.touchActionTaken();
    }

    public void run() {
        checkMouse();
    }


    //------------------------------------------------------------
    //- Inner Classes

    private static class Runner implements Runnable {
        public void run() {
            INSTANCE.checkMouse();
        }
    }


    //------------------------------------------------------------
    //- Main


}
