/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.gui;

import org.apache.log4j.Logger;

import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;
import java.awt.*;

import com.notuvy.thread.IdlenessMonitorable;

/**
 * An EventQueue that keeps track of last activity, so it can be determined
 * how long the user is idle.
 *
 * http://www.javakb.com/Uwe/Forum.aspx/java-gui/2628/Inactivity-detect
 *
 * User: murali
 * Date: Aug 29, 2008
 * Time: 1:02:12 PM
 */
public class SwingActivityEventQueue extends EventQueue implements IdlenessMonitorable {

    //------------------------------------------------------------
    //- Class Variables

    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger LOG = Logger.getLogger("com.notuvy.gui");

    private static final SwingActivityEventQueue INSTANCE = new SwingActivityEventQueue();

    static {
        EventQueue current = Toolkit.getDefaultToolkit().getSystemEventQueue();
        if (current != INSTANCE) {
            current.push(INSTANCE);
        }
    }


    //------------------------------------------------------------
    //- Class Functions

    public static SwingActivityEventQueue getInstance() {
        return INSTANCE;
    }


    //------------------------------------------------------------
    //- Instance Variables

    private final UIActivityMonitor fActivityMonitor = new UIActivityMonitor();

    //------------------------------------------------------------
    //- Constructors

    private SwingActivityEventQueue() {
    }

    //------------------------------------------------------------
    //- Getters


    //------------------------------------------------------------
    //- Settors

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    //------------------------------------------------------------
    //- Public Interface Functions

    //------------------------------------------------------------
    //- Class Interface Functions

    @Override
    public String toString() {
        return String.format("Swing activity [%s].", fActivityMonitor);
    }

    public long lastActivity() {
        return fActivityMonitor.lastActivity();
    }

    public void markActionTakenOnIdle() {
        fActivityMonitor.touchActionTaken();
    }

    protected void dispatchEvent(AWTEvent event) {
        try {
            super.dispatchEvent(event);
        } catch (RuntimeException re) {
            LOG.error(String.format("UI error dispatching [%s].", event.toString()), re);
            throw re;
        }

        // Note: maybe we want KEY_PRESSED, because when a button launches an
        // action that consumes thread, the release does not register until after
        // the action.
        switch (event.getID()) {
            case MouseEvent.MOUSE_RELEASED:
            case KeyEvent.KEY_RELEASED:
                fActivityMonitor.touchUIActivity();
                break;
        }
    }

    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main

}