/*
   Copyright 2007-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.gui;

import java.awt.Component;

import javax.swing.JDialog;
import javax.swing.JOptionPane;

/**
 * A warning dialog that is displayed only for a limited amount of time.
 *
 * @author  murali
 */
public class TimedNowLaterDialog extends TimedDialog {

    //------------------------------------------------------------
    //- Class Variables

    private static final String OPTION_NOW = "Now";
    private static final String OPTION_LATER = "Later";
    private static final Object[] OPTIONS = {OPTION_NOW, OPTION_LATER};


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final JOptionPane vPane;


    //------------------------------------------------------------
    //- Constructors

    public TimedNowLaterDialog(Component pParent, String pTitle, String pMessage, int pSeconds) {
        super (pParent, pTitle, pMessage, (pSeconds * 1000));

        vPane = new JOptionPane(pMessage, JOptionPane.QUESTION_MESSAGE, JOptionPane.DEFAULT_OPTION, null,  OPTIONS);
    }


    //------------------------------------------------------------
    //- Accessors

    private JOptionPane getPane() {
        return vPane;
    }



    //------------------------------------------------------------
    //- Settors



    //------------------------------------------------------------
    //- Private/Protected Utility Functions



    //------------------------------------------------------------
    //- Public Interface Functions

    public boolean isNow() {
        boolean result = getPane().getValue() == OPTION_NOW;
        return result;
    }

    public boolean isLater() {
        boolean result = getPane().getValue() == OPTION_LATER;
        return result;
    }

    public boolean isUnselected() {
        boolean result = (getPane().getValue() == null) || (getPane().getValue() == JOptionPane.UNINITIALIZED_VALUE);
        return result;
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    protected JOptionPane createOptionPane() {
        JOptionPane result = getPane();

        return result;
    }


    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main

    public static void main(String[] args) {
        try {
            TimedNowLaterDialog dialog = new TimedNowLaterDialog(null, "Title", "Do action now or later?", 10);
            dialog.show();
            LOG.error("<now,later,unselected>: <" + dialog.isNow() + ", " + dialog.isLater() + ", " + dialog.isUnselected() + ">");
            System.exit(0);
        } catch (Throwable thr) {
            LOG.error("Fatal error.", thr);
        }
    }


}
