package com.notuvy.gui;

import org.apache.log4j.Logger;
import com.notuvy.util.WallClock;

import java.util.Date;

/**
 * The "latest activity" is the later of the UI activity, or the action taken due to UI idleness.
 * This represents those two values separately, and does the "latest" calculation.
 *
 * @author murali
 */
public class UIActivityMonitor {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger("com.notuvy.gui");


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private long fLastUIActivity = 0L;
    private long fLastActionTaken = WallClock.currentTimeMillis();


    //------------------------------------------------------------
    //- Constructors


    //------------------------------------------------------------
    //- Gettors

    public long getLastUIActivity() {
        return fLastUIActivity;
    }

    public long getLastActionTaken() {
        return fLastActionTaken;
    }


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected void touchUIActivity() {
        fLastUIActivity = WallClock.currentTimeMillis();
    }

    protected void touchActionTaken() {
        fLastActionTaken = WallClock.currentTimeMillis();
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    // The last activity is whichever is more recent, the last UI activity or
    // the last action taken by the IdleMonitor.
    public long lastActivity() {
        return Math.max(getLastUIActivity(), getLastActionTaken());
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    @Override
    public String toString() {
        return String.format("UI [%s], Action [%s].", new Date(getLastUIActivity()), new Date(getLastActionTaken()));
    }


    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main


}
