/*
   Copyright 2004-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.image;

import java.awt.*;
import java.util.*;

import javax.swing.*;

import org.apache.log4j.Logger;

/**
 * GUI component to display an image.
 * 
 * @author  murali
 * @version  $Id: ImageDisplay.java,v 1.2 2004/09/14 15:01:10 murali Exp $
 * @since  Rearden
 */
public class ImageDisplay extends JComponent {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(ImageDisplay.class);


    //------------------------------------------------------------
    //- Class Functions



    //------------------------------------------------------------
    //- Instance Variables

    private Image vImage = null;


    //------------------------------------------------------------
    //- Constructors

    protected ImageDisplay() {
    }


    public ImageDisplay(Image pImage) {
        setImage(pImage);
    }


    //------------------------------------------------------------
    //- Accessors

    public Image getImage() {
        return vImage;
    }


    //------------------------------------------------------------
    //- Settors

    protected final void setImage(Image pImage) {
        vImage = pImage;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions



    //------------------------------------------------------------
    //- Public Interface Functions

    public String toString() {
        String result = "ImageDisplay[]";
        return result;
    }


    public void reset(Image pImage) {
        setImage(pImage);
        invalidate();
        repaint();
    }



    //------------------------------------------------------------
    //- Class Interface Functions

    public void paint(Graphics graphics) {
        int xStart = 0;
        int yStart = 0;

        Rectangle displayArea = graphics.getClipBounds();
        int imageWidth = getImage().getWidth(null);
        int imageHeight = getImage().getHeight(null);

        if (displayArea.width > imageWidth) {
            xStart = (displayArea.width - imageWidth) / 2;
        }

        if (displayArea.height > imageHeight) {
            yStart = (displayArea.height - imageHeight) / 2;
        }

        graphics.drawImage(getImage(), xStart, yStart, null);
    }


    public Dimension getPreferredSize() {
        Dimension result = new Dimension(0, 0);

        if (getImage() != null) {
            int wImage = getImage().getWidth(null);
            int hImage = getImage().getHeight(null);

            result = new Dimension(wImage, hImage);
        }

        return result;
    }


    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main



}
