/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.image;

import org.apache.log4j.Logger;

/**
 * A enumeration representing the orientation of an image.  It is used
 * to keep track of the state of an image after rotation.
 *
 * @author  murali
 */
public abstract class Orientation {

    //------------------------------------------------------------
    //- Class Variables

    public static final Orientation UP = new Up();
    public static final Orientation RIGHT = new Right();
    public static final Orientation DOWN = new Down();
    public static final Orientation LEFT = new Left();


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables


    //------------------------------------------------------------
    //- Constructors

    private Orientation() {
    }


    //------------------------------------------------------------
    //- Accessors


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected boolean equalsUp(Orientation pOrientation) {
        return false;
    }

    protected boolean equalsRight(Orientation pOrientation) {
        return false;
    }

    protected boolean equalsDown(Orientation pOrientation) {
        return false;
    }

    protected boolean equalsLeft(Orientation pOrientation) {
        return false;
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public boolean equals(Object object) {
        boolean result = false;

        if (object instanceof Orientation) {
            result = equalsOrientation((Orientation) object);
        }

        return result;
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    public boolean isUp() {
        return false;
    }

    public boolean isRight() {
        return false;
    }

    public boolean isDown() {
        return false;
    }

    public boolean isLeft() {
        return false;
    }

    public abstract Orientation rotateRight();
    public abstract Orientation rotateLeft();
    protected abstract boolean equalsOrientation(Orientation pOrientation);


    //------------------------------------------------------------
    //- Inner Classes

    public static class Up extends Orientation {

        protected boolean equalsOrientation(Orientation pOrientation) {
            return pOrientation.equalsUp(this);
        }

        protected boolean equalsUp(Orientation pOrientation) {
            return true;
        }
        
        public Orientation rotateRight() {
            return RIGHT;
        }
        
        public Orientation rotateLeft() {
            return LEFT;
        }

        public String toString() {
            return "Orientation[Up]";
        }

        public boolean isUp() {
            return true;
        }

    }


    public static class Right extends Orientation {

        protected boolean equalsOrientation(Orientation pOrientation) {
            return pOrientation.equalsRight(this);
        }

        protected boolean equalsRight(Orientation pOrientation) {
            return true;
        }
        
        public Orientation rotateRight() {
            return DOWN;
        }
        
        public Orientation rotateLeft() {
            return UP;
        }

        public String toString() {
            return "Orientation[Right]";
        }

        public boolean isRight() {
            return true;
        }
        
    }


    public static class Down extends Orientation {

        protected boolean equalsOrientation(Orientation pOrientation) {
            return pOrientation.equalsDown(this);
        }

        protected boolean equalsDown(Orientation pOrientation) {
            return true;
        }
        
        public Orientation rotateRight() {
            return LEFT;
        }
        
        public Orientation rotateLeft() {
            return RIGHT;
        }

        public String toString() {
            return "Orientation[Down]";
        }

        public boolean isDown() {
            return true;
        }
        
    }


    public static class Left extends Orientation {

        protected boolean equalsOrientation(Orientation pOrientation) {
            return pOrientation.equalsLeft(this);
        }

        protected boolean equalsLeft(Orientation pOrientation) {
            return true;
        }
        
        public Orientation rotateRight() {
            return UP;
        }
        
        public Orientation rotateLeft() {
            return DOWN;
        }

        public String toString() {
            return "Orientation[Left]";
        }

        public boolean isLeft() {
            return true;
        }
        
    }


    //------------------------------------------------------------
    //- Main



}
