/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.net;

import org.apache.commons.lang.text.StrBuilder;
import org.apache.log4j.Logger;

import java.util.Comparator;
import java.util.Date;

/**
 * Created by IntelliJ IDEA.
 * User: murali
 * Date: Sep 11, 2008
 * Time: 12:35:32 PM
 */
public enum CacheSortOrder {

    CREATION("Creation Time", "creation") {
        public Comparator<CachedItem> comparator() {
            return CachedItem.BY_CREATION;
        }},
    LAST_ACCESS("Last Access Time", "lastaccess") {
        public Comparator<CachedItem> comparator() {
            return CachedItem.BY_LAST_ACCESS;
        }

        public Date extractTime(CachedItem pItem) {
            return pItem.getLastAccessed();
        }},
    URL("URL", "url") {
        public Comparator<CachedItem> comparator() {
            return CachedItem.BY_URL;
        }};

    private final String fName;
    private final String fCode;

    private CacheSortOrder(String pName, String pCode) {
        fName = pName;
        fCode = pCode;
    }

    public String getName() {
        return fName;
    }

    public String getCode() {
        return fCode;
    }

    public abstract Comparator<CachedItem> comparator();

    public Date extractTime(CachedItem pItem) {
        return pItem.getCreated();
    }
    public static CacheSortOrder lookup(String pCode) {
        CacheSortOrder result = null;  // Pessimistic.
        StrBuilder possibilities = new StrBuilder();
        for (CacheSortOrder order : values()) {
            possibilities.appendSeparator(',').append(order.getCode());
            if (order.getCode().equals(pCode)) {
                result = order;
            }
        }
        if (result == null) {
            Logger.getLogger("com.notuvy.net").error("[" + pCode + "] does not match any of [" + possibilities + "]");
        }
        return result;
    }
}
