/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.net;

import org.apache.log4j.Logger;
import org.apache.commons.lang.StringUtils;

import java.net.URLEncoder;
import java.io.UnsupportedEncodingException;

/**
 * Create a GET URL by iteratively adding the parameters.
 *
 * User: murali
 * Date: Aug 12, 2008
 * Time: 11:54:17 AM
 * To change this template use File | Settings | File Templates.
 */
public class UrlBuilder {

    //------------------------------------------------------------
    //- Class Variables

    private static final Logger LOG = Logger.getLogger(UrlBuilder.class);

    //------------------------------------------------------------
    //- Class Functions

    //------------------------------------------------------------
    //- Instance Variables

    private final String fBase;
    private StringBuilder fArgs = new StringBuilder();

    //------------------------------------------------------------
    //- Constructors

    public UrlBuilder(String pBase) {
        fBase = pBase;
    }

    //------------------------------------------------------------
    //- Getters

    //------------------------------------------------------------
    //- Settors

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    //------------------------------------------------------------
    //- Public Interface Functions

    public UrlBuilder add(String pKey, Object pVal) {
        fArgs.append((fArgs.length() == 0) ? '?' : '&');
        fArgs.append(pKey).append('=');
        String value = String.valueOf(pVal);
        try {
            fArgs.append(URLEncoder.encode(value, "UTF-8"));
        } catch (UnsupportedEncodingException uee) {
            LOG.error("URL encoding error.", uee);
            fArgs.append(value);
        }
        return this;
    }

    //------------------------------------------------------------
    //- Class Interface Functions

    public String toString() {
        return fBase + fArgs;
    }

    //------------------------------------------------------------
    //- Inner Classes

    //------------------------------------------------------------
    //- Main

}
