/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.net;

import java.io.*;
import java.util.*;

import org.apache.commons.httpclient.*;
import org.apache.commons.httpclient.methods.*;

import org.apache.log4j.Logger;

/**
 * Take the given url and follow all the redirects until it reaches a final resource.
 *
 * @author  murali
 */
public class UrlRedirectResolve {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(UrlRedirectResolve.class);
    private boolean fPerformed = false;


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final LinkedList<String> fSequence = new LinkedList<String>();


    //------------------------------------------------------------
    //- Constructors

    public UrlRedirectResolve(String pUrl) {
        getSequence().add(pUrl);
    }


    //------------------------------------------------------------
    //- Accessors

    public LinkedList<String> getSequence() {
        return fSequence;
    }

    public String getUrl() {
        return getSequence().getFirst();
    }


    //------------------------------------------------------------
    //- Settors



    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected String findRedirect() {
        String result = null;

        HttpClient client = new HttpClient();
        HeadMethod method = new HeadMethod(getSequence().getLast());
        method.setFollowRedirects(false);
        try {
            int statusCode = client.executeMethod(method);
            if ((statusCode == HttpStatus.SC_MOVED_PERMANENTLY) || (statusCode == HttpStatus.SC_MOVED_TEMPORARILY) || (statusCode == HttpStatus.SC_TEMPORARY_REDIRECT)) {
                Header header = method.getResponseHeader("Location");
                if (header != null) {
                    result = header.getValue();
                }
            }
        } catch (IOException ioe) {
            LOG.error("Problem downloading.", ioe);
        } catch (Exception ex) {
            LOG.error("Basic downloading problem.", ex);
        } finally {
            method.releaseConnection();
        }

        return result;
    }

    //------------------------------------------------------------
    //- Public Interface Functions

    public String resolve() {
        if (!fPerformed) {
            String candidate = findRedirect();
            while (candidate != null) {
                getSequence().add(candidate);
                candidate = findRedirect();
            }

            fPerformed = true;
        }

        return getSequence().getLast();
    }


    //------------------------------------------------------------
    //- Class Interface Functions



    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main

    public static void main(String[] args) {
        try {
            for (String arg : args) {
                UrlRedirectResolve resolve = new UrlRedirectResolve(arg);
                LOG.info("from [" + resolve.getUrl() + "] to [" + resolve.resolve() + "].");
            }
        } catch (Throwable thr) {
            LOG.error("Fatal error.", thr);
        }
    }


}
