package com.notuvy.podcast;

import org.jdom.Element;
import org.jdom.Namespace;

import java.util.ArrayList;
import java.util.List;

/**
 * The RSS/channel object and the container of the PodcastItem objects.
 *
 * @author murali
 */
public class PodcastChannel extends PodcastBase {

    //------------------------------------------------------------
    //- Class Variables

    public static final String XML_Node_rss = "rss";
    private static final String XML_Node_channel = "channel";
    private static final String XML_Node_title = "title";
    private static final String XML_Node_link = "link";
    private static final String XML_Node_desc = "description";


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final List<PodcastItem> fItems = new ArrayList<PodcastItem>();


    //------------------------------------------------------------
    //- Constructors

    protected PodcastChannel() {
    }

    public PodcastChannel(String pTitle, String pLink, String pDesc) {
        super (pTitle, pLink, pDesc);
    }

    //------------------------------------------------------------
    //- Accessors

    public List<PodcastItem> getItems() {
        return fItems;
    }

    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    public PodcastItem addPodcastItem(String pTitle, String pLink, String pDesc, String pDuration, String pDate, String pGuid, String pEnclosureUrl, String pEnclosureLength) {
        PodcastItem result = new PodcastItem(pTitle, pLink, pDesc, pDuration, pDate, pGuid, pEnclosureUrl, pEnclosureLength);
        getItems().add(result);
        return result;
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    @Override
    public String rootName() {
        return XML_Node_rss;
    }

    protected Element toXml() {
        Element result = createRoot();
        if (gNsItunes == null) {
            gNsItunes = Namespace.getNamespace("itunes", "http://www.itunes.com/dtds/podcast-1.0.dtd");
        }
        result.addNamespaceDeclaration(gNsItunes);

        Element channel = new Element(XML_Node_channel);
        channel.addContent(new Element(XML_Node_title).setText(getTitle()));
        channel.addContent(new Element(XML_Node_link).setText(getSiteUrl()));
        channel.addContent(new Element(XML_Node_desc).setText(getDescription()));
        for (PodcastItem item : getItems()) {
            channel.addContent(item.toXml());
        }
        result.addContent(channel);
        return result;
    }

    protected void parse(Element pRoot) {
        Element channel = pRoot.getChild(XML_Node_channel);
        for (Object obj : pRoot.getAdditionalNamespaces()) {
            Namespace candidate = (Namespace) obj;
            if ("itunes".equals(candidate.getPrefix())) {
                gNsItunes = candidate;
            }
        }

        if (channel == null) {
            LOG.error("No <channel> under <rss>");
        } else {
            setTitle(getTextFromElement(channel.getChild(XML_Node_title)));
            setSiteUrl(getTextFromElement(channel.getChild(XML_Node_link)));
            setDescription(getTextFromElement(channel.getChild(XML_Node_desc)));

            @SuppressWarnings({"unchecked"}) List<Element> itemElements = channel.getChildren(PodcastItem.XML_Node_item);
            for (Element itemElement : itemElements) {
                PodcastItem item = new PodcastItem(itemElement);
                if (item.isValid()) {
                    getItems().add(item);
                }
            }
        }
    }

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder("Podcast");
        builder.append("[").append(getTitle()).append("] {\n");
        for (PodcastItem item : getItems()) {
            builder.append("  ").append(item).append("\n");
        }
        builder.append("}");
        return builder.toString();
    }

    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main

}
