package com.notuvy.podcast;

import org.apache.log4j.Logger;

import java.io.*;
import java.net.URL;
import java.net.MalformedURLException;

/**
 * Logic to extract a PodcastChannel from any of the possible sources.
 * This is basically a factory.
 *
 * @author murali
 */
public class PodcastFactory {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger("com.notuvy.podcast");


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables


    //------------------------------------------------------------
    //- Constructors


    //------------------------------------------------------------
    //- Accessors


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    public PodcastChannel fromFilename(String pFilename) {
        return fromFile(new File(pFilename));
    }

    public PodcastChannel fromFile(File pFile) {
        PodcastChannel pResult = new PodcastChannel();
        try {
            FileReader reader = new FileReader(pFile);
            pResult.readFrom(reader);
            reader.close();
        } catch (IOException ioe) {
            LOG.error("Problem reading file", ioe);
            pResult = null;
        }
        return pResult;
    }

    public PodcastChannel fromUrlString(String pUrlString) {
        PodcastChannel result = null;
        try {
            result = fromURL(new URL(pUrlString));
        } catch (MalformedURLException mue) {
            LOG.error(String.format("Invalid url [%s]", pUrlString), mue);
        }
        return result;
    }

    public PodcastChannel fromURL(URL pUrl) {
        PodcastChannel result = new PodcastChannel();
        try {
            InputStream is = pUrl.openStream();
            InputStreamReader reader = new InputStreamReader(is);
            result.readFrom(reader);
            reader.close();
            is.close();
        } catch (IOException e) {
            LOG.error("Problem reading url", e);
        }
        return result;
    }

    public PodcastChannel from(String pGenericString) {
        PodcastChannel result = null;
        File file = new File(pGenericString);
        if (file.exists()) {
            result = fromFile(file);
        } else if (pGenericString.startsWith("http:")) {
            result = fromUrlString(pGenericString);
        } else {
            LOG.error(String.format("Invalid file or url [%s]", pGenericString));
        }
        return result;
    }

    public PodcastChannel fromStringContent(String pString) {
        PodcastChannel result = new PodcastChannel();
        StringReader reader = new StringReader(pString);
        result.readFrom(reader);
        return result;
    }

    //------------------------------------------------------------
    //- Class Interface Functions


    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main

    public static void main(String[] args) {
        try {
//            PodcastChannel podcast = fromFile("/var/tmp/feed_source.xml");
//            PodcastChannel podcast = fromUrlString("http://feeds.thisamericanlife.org/talpodcast");
//            PodcastChannel podcast = fromUrlString("http://www.daveslounge.com/rss");
            PodcastFactory factory = new PodcastFactory();
            for (String arg : args) {
                LOG.info(factory.from(arg));
            }
        } catch (Throwable thr) {
            LOG.error("Fatal error", thr);
        }
    }


}
