package com.notuvy.thread;

import org.apache.log4j.Logger;

import java.util.List;
import java.util.ArrayList;
import java.util.Arrays;

/**
 * A Runnable that is a list of Runnables.  Delegate to the children.
 *
 * @author murali
 */
public class MultiRunnable implements Runnable {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger("com.notuvy.thread");


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final List<Runnable> fChildren = new ArrayList<Runnable>();


    //------------------------------------------------------------
    //- Constructors

    public MultiRunnable(Runnable...pChildren) {
        fChildren.addAll(Arrays.asList(pChildren));
    }


    //------------------------------------------------------------
    //- Gettors


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    public MultiRunnable add(Runnable pChild) {
        fChildren.add(pChild);

        return this;
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public void run() {
        for (Runnable child : fChildren) {
            try {
                child.run();
            } catch (Throwable thr) {
                LOG.error(String.format("Fatal error at top-level for runnable: %s", child), thr);
            }
        }
    }


    //------------------------------------------------------------
    //- Class Interface Functions


    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main


}
