package com.notuvy.thread;

import org.apache.log4j.Logger;

import java.util.concurrent.ThreadFactory;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * A simple ThreadFactory that sets an explicit name for the Thread and also defaults it
 * to be a daemon and sets a simple uncaught exception handler.
 *
 * @author murali
 */
public class NamedThreadFactory implements ThreadFactory {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger("com.notuvy.thread");


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final AtomicInteger fIndex = new AtomicInteger(0);
    private final String fName;
    private final boolean fDaemon;
    private final Thread.UncaughtExceptionHandler fHandler;


    //------------------------------------------------------------
    //- Constructors

    public NamedThreadFactory(String pName, boolean pDaemon, Thread.UncaughtExceptionHandler pHandler) {
        fName = pName;
        fDaemon = pDaemon;
        fHandler = pHandler;
    }

    public NamedThreadFactory(String pName, boolean pDaemon) {
        this (
                pName,
                pDaemon,
                new Thread.UncaughtExceptionHandler() {
                    public void uncaughtException(Thread pThread, Throwable pThrowable) {
                        LOG.error("Fatal thread error.", pThrowable);
                    }
                }
        );
    }

    public NamedThreadFactory(String pName) {
        this (pName, true);
    }


    //------------------------------------------------------------
    //- Gettors


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions


    //------------------------------------------------------------
    //- Class Interface Functions

    public Thread newThread(Runnable pRunnable) {
        Thread result = new Thread(pRunnable, String.format("%s-%d", fName, fIndex.incrementAndGet()));
        result.setDaemon(fDaemon);
        result.setUncaughtExceptionHandler(fHandler);

        return result;
    }


    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main


}
