package com.notuvy.util;

import java.util.*;


/**
 * Calculation of estimated completion time.
 *
 * User: murali
 * Date: Apr 16, 2009
 * Time: 2:49:12 PM
 */
public class CompletionTime {

    //------------------------------------------------------------
    //- Class Variables



    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private long fStartTime;

    //------------------------------------------------------------
    //- Constructors

    public CompletionTime() {
        this (WallClock.currentTimeMillis());
    }

    public CompletionTime(long pStartTime) {
        fStartTime = pStartTime;
    }

    //------------------------------------------------------------
    //- Getters

    public long getStartTime() {
        return fStartTime;
    }

    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions



    //------------------------------------------------------------
    //- Public Interface Functions

    public long completionTimeFor(Fractional pFractionComplete) {
        long pCurrentTime = WallClock.currentTimeMillis();
        long result;
        long elapsed = pCurrentTime - getStartTime();
        if ((elapsed == 0) || (pFractionComplete.isZero())) {
            result = pCurrentTime;
        } else {
            double total = elapsed / pFractionComplete.getValue();
            result = Math.round(total) + getStartTime();
        }
        return result;
    }

    public Date completionDateFor(Fractional pFractionComplete) {
        return new Date(completionTimeFor(pFractionComplete));
    }

    public String remaining(Fractional pFractionComplete) {
        long pCurrentTime = WallClock.currentTimeMillis();
        StringBuilder builder = new StringBuilder();
        long elapsed = pCurrentTime - getStartTime();
        if ((elapsed == 0) || (pFractionComplete.isZero())) {
            builder.append("unknown");
        } else {
            double total = elapsed / pFractionComplete.getValue();
            long remaining = Math.round(total) - elapsed;

            // Now break it into the hh:mm:ss parts.
            LinkedList<String> parts = new LinkedList<String>();
            if (remaining < 1000) {
                parts.add("1s");
            } else {
                long totalSeconds = remaining / 1000;
                long seconds = totalSeconds % 60;
                parts.add(seconds+"s");
                long totalMinutes = totalSeconds / 60;
                if (totalMinutes > 0) {
                    long minutes = totalMinutes % 60;
                    parts.add(minutes+"m");
                    long totalHours = totalMinutes / 60;
                    if (totalHours > 0) {
                        parts.add(totalHours+"h");
                    }
                }
            }
            Collections.reverse(parts);
            while (parts.size() > 2) {
                parts.removeLast();
            }

            for (String part : parts) {
                if (builder.length() > 0) {
                    builder.append(" ");
                }
                builder.append(part);
            }
        }
        return builder.toString();
    }

    //------------------------------------------------------------
    //- Class Interface Functions


    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main

}
