/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.util;

import org.apache.commons.lang.text.StrTokenizer;

/**
 * Break long text with word-wrap to lines of maximum length.
 *
 * @author murali
 */
public class FixedWidthText {

    //------------------------------------------------------------
    //- Class Variables

    // protected static final Logger LOG = Logger.getLogger("com.notuvy.util");


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final int fMaxWidth;

    //------------------------------------------------------------
    //- Constructors

    public FixedWidthText(int pMaxWidth) {
        fMaxWidth = pMaxWidth;
    }

    //------------------------------------------------------------
    //- Accessors

    public int getMaxWidth() {
        return fMaxWidth;
    }

    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    public String process(String pInput) {
        boolean firstLine = true;
        StringBuilder builder = new StringBuilder();
        StringBuilder current = new StringBuilder();
        StrTokenizer tokenizer = new StrTokenizer(pInput);
        while (tokenizer.hasNext()) {
            String nextWord = tokenizer.nextToken();
            if ((current.length() + nextWord.length() + 1) > getMaxWidth()) {
                if (firstLine) {
                    firstLine = false;
                } else {
                    builder.append("\n");
                }
                builder.append(current);
                current = new StringBuilder(nextWord);
            } else {
                if (current.length() > 0) {
                    current.append(" ");
                }
                current.append(nextWord);
            }
        }
        builder.append("\n").append(current);
        return builder.toString();
    }

    //------------------------------------------------------------
    //- Class Interface Functions


    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main


}
