/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.util;

import java.util.*;

/**
 * Simple rot-13 obfuscation.
 *
 * @author  murali
 * @version  $Id: HashCrypt.java,v 1.2 2006/06/17 06:43:33 murali Exp $
 */
public class HashCrypt {

    //------------------------------------------------------------
    //- Class Variables

    private static final int START = 32;
    private static final int SIZE = 95;


    //------------------------------------------------------------
    //- Class Functions



    //------------------------------------------------------------
    //- Instance Variables

    private char[] vForward = new char[SIZE];
    private char[] vReverse = new char[SIZE];


    //------------------------------------------------------------
    //- Constructors

    public HashCrypt(long pSeed) {
        Random random = new Random(pSeed);
        List list = new ArrayList(SIZE);
        for (int i = 0; i < SIZE; i++) {
            list.add(new Character((char) (i + START)));
        }
        Collections.shuffle(list, random);
        Iterator iterator = list.iterator();
        int i = 0;
        while (iterator.hasNext()) {
            Character character = (Character) iterator.next();
            char ch = character.charValue();
            int index = i++;
            getForward()[index] = ch;
            getReverse()[((int) ch) - START] = (char) (index + START);
        }
    }


    public HashCrypt(Object pObject) {
        this (pObject.hashCode());
    }


    //------------------------------------------------------------
    //- Accessors

    protected char[] getForward() {
        return vForward;
    }

    protected char[] getReverse() {
        return vReverse;
    }



    //------------------------------------------------------------
    //- Settors



    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected char translate(char in, char[] map) {
        char result = in;

        int index = ((int) in) - START;
        if ((index >= 0) && (index < SIZE)) {
            result = map[index];
        }

        return result;
    }


    protected char encrypt(char in) {
        char result = translate(in, getForward());
        return result;
    }


    protected char decrypt(char in) {
        char result = translate(in, getReverse());
        return result;
    }



    //------------------------------------------------------------
    //- Public Interface Functions

    public String toString() {
        StringBuffer buffer = new StringBuffer();

        for (int i = 0; i < SIZE; i++) {
            int trans = (i + START);
            char crypted = getForward()[i];
            buffer.append("[" + trans + "] '" + ((char) trans) + "' '" + crypted + "' '" + getReverse()[((int)crypted) - START] + "'\n");
        }

        return buffer.toString();
    }


    public String encrypt(String pString) {
        String result = null;  // Pessimistic.

        if (pString != null) {
            char[] chars = pString.toCharArray();
            char[] translated = new char[chars.length];
            for (int i = 0; i < chars.length; i++) {
                translated[i] = encrypt(chars[i]);
            }

            result = new String(translated);
        }

        return result;
    }


    public String decrypt(String pString) {
        String result = null;  // Pessimistic.

        if (pString != null) {
            char[] chars = pString.toCharArray();
            char[] translated = new char[chars.length];
            for (int i = 0; i < chars.length; i++) {
                translated[i] = decrypt(chars[i]);
            }

            result = new String(translated);
        }

        return result;
    }


    //------------------------------------------------------------
    //- Class Interface Functions



    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main



}
