/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.util;



import java.util.HashMap;
import java.net.URL;

import javax.swing.ImageIcon;



/**
 * Read a resource from the jar (classpath actually) and return it as an icon.
 * 
 * @author  Murali Krishnan
 * @version  $Id: IconResource.java,v 1.1 2006/06/02 04:11:52 murali Exp $
 */
public class IconResource extends PackageResource {     

    //------------------------------------------------------------  
    //- Class Variables     

    private static final HashMap sCache = new HashMap();



    //------------------------------------------------------------  
    //- Class Functions     

    public static ImageIcon get(String pUri) {
        ImageIcon result = (ImageIcon) sCache.get(pUri);

        if (result == null) {
            IconResource resource = new IconResource(pUri);
            result = resource.getIcon();
            sCache.put(pUri, result);
        }

        return result;
    }



    //------------------------------------------------------------  
    //- Instance Variables  



    //------------------------------------------------------------  
    //- Constructors        

    protected IconResource(String pUri) {
        super(pUri);
    }


    //------------------------------------------------------------  
    //- Accessors   



    //------------------------------------------------------------  
    //- Settors     



    //------------------------------------------------------------  
    //- Private/Protected Utility Functions 

    protected synchronized Object readContent() {
        URL resource = getClass().getResource(getUri());
        if (resource == null) {
            // Check deprecated location (pre-maven ant builds) for backward compatibility.
            LOG.warn("Found resource in deprecated (/include) location.");
            resource = getClass().getResource("/include"+getUri());
        }
        ImageIcon result = new ImageIcon(resource);

        return result;
    }



    //------------------------------------------------------------  
    //- Public Interface Functions  

    public ImageIcon getIcon() {
        ImageIcon result = (ImageIcon) getContent();

        return result;
    }



    //------------------------------------------------------------  
    //- Class Interface Functions   



    //------------------------------------------------------------  
    //- Inner Classes       



    //------------------------------------------------------------  
    //- Main        



}       
