/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.util;



import java.util.HashMap;
import java.net.URL;
import java.awt.*;
import java.io.IOException;

import javax.imageio.ImageIO;


/**
 * Read a resource from the jar (classpath actually) and return it as an Image.
 *
 * @author  Murali Krishnan
 */
public class ImageResource extends PackageResource {

    //------------------------------------------------------------
    //- Class Variables

    private static final HashMap<String,Image> sCache = new HashMap<String,Image>();



    //------------------------------------------------------------
    //- Class Functions

    public static Image get(String pUri) {
        Image result = sCache.get(pUri);

        if (result == null) {
            ImageResource resource = new ImageResource(pUri);
            result = resource.getImage();
            sCache.put(pUri, result);
        }

        return result;
    }



    //------------------------------------------------------------
    //- Instance Variables



    //------------------------------------------------------------
    //- Constructors

    protected ImageResource(String pUri) {
        super(pUri);
    }


    //------------------------------------------------------------
    //- Accessors



    //------------------------------------------------------------
    //- Settors



    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected synchronized Object readContent() {
        URL resource = getClass().getResource(getUri());
        if (resource == null) {
            // Check deprecated location (pre-maven ant builds) for backward compatibility.
            LOG.warn("Found resource in deprecated (/include) location.");
            resource = getClass().getResource("/include"+getUri());
        }
        Image result = null;
        try {
            result = ImageIO.read(resource);
        } catch (IOException ioe) {
            LOG.error(String.format("Error reading resource [%s].", getUri()), ioe);
        }

        return result;
    }



    //------------------------------------------------------------
    //- Public Interface Functions

    public Image getImage() {
        return (Image) getContent();
    }



    //------------------------------------------------------------
    //- Class Interface Functions



    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main


}