/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.util;

import org.apache.log4j.Logger;
import org.apache.commons.lang.StringUtils;
import com.notuvy.file.Directory;

/**
 * The framework for setting up the context for a standalone application.
 * It sets system properties that define the application directory, and it
 * initializes the logging.
 *
 * It must be immutable.
 *
 * User: murali
 * Date: Jan 14, 2009
 * Time: 9:28:35 AM
 */
public class MainContext {

    //------------------------------------------------------------
    //- Class Variables

    public static final String APP_PROPERTY_LOG_NAME = "app.log.file";


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final Directory fAppDir;
    private final Logger fMainLogger;

    //------------------------------------------------------------
    //- Constructors

    public MainContext(String pSubDir, Class pReferenceClass) {
        // Set the app.home BEFORE loading the logger.  The log config depends on the property.
        fAppDir = Directory.ensureInHome(pSubDir).setAppHome();

        // Set the log name BEFORE loading the logger.  The log config depends on the property.
        // If the system property was already set, just use that.
        if (StringUtils.isBlank(System.getProperty(APP_PROPERTY_LOG_NAME))) {
            String filename = pReferenceClass.getSimpleName().toLowerCase() + ".log";
            System.setProperty(APP_PROPERTY_LOG_NAME, filename);
        }

        fMainLogger = Logger.getLogger(pReferenceClass.getPackage().getName());
    }

    //------------------------------------------------------------
    //- Getters

    public Directory getAppDir() {
        return fAppDir;
    }

    public Logger getMainLogger() {
        return fMainLogger;
    }

    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions


    //------------------------------------------------------------
    //- Class Interface Functions


    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main

}
