/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.util;

import java.util.List;
import java.util.ArrayList;

/**
 * Parse a multiline string into a list of string lines.
 *
 * User: murali
 * Date: Jan 2, 2008
 * Time: 2:52:36 PM
 */
public class MultilineString {

    //------------------------------------------------------------
    //- Class Variables


    //------------------------------------------------------------
    //- Class Functions

    public static List<String> toLinesList(String pOriginal) {
        return toLinesList(pOriginal, false);
    }

    public static List<String> toLinesList(String pOriginal, boolean pTrim) {
        MultilineString strings = new MultilineString(pOriginal, pTrim);
        return strings.getList();
    }


    //------------------------------------------------------------
    //- Instance Variables

    private final List<String> fList;


    //------------------------------------------------------------
    //- Constructors

    public MultilineString(String pOriginal) {
        this (pOriginal, false);
    }

    public MultilineString(String pOriginal, boolean pTrim) {
        String[] lines = pOriginal.split("\\n");
        List<String> list = new ArrayList<String>(lines.length);
        for (String line : lines) {
            list.add(pTrim ? line.trim() : line);
        }
        fList = list;
    }


    //------------------------------------------------------------
    //- Getters

    public List<String> getList() {
        return fList;
    }

    //------------------------------------------------------------
    //- Settors

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    //------------------------------------------------------------
    //- Public Interface Functions

    //------------------------------------------------------------
    //- Class Interface Functions

    //------------------------------------------------------------
    //- Inner Classes

    //------------------------------------------------------------
    //- Main

}
