/*
   Copyright 2007-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.util;

import java.util.*;
import java.util.regex.*;

import org.apache.log4j.Logger;

/**
 * Match a string against multiple patterns.  Try the patterns in order,
 * and handle the first match.
 *
 * @author  murali
 */
public class MultiplePatternMatcher {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(MultiplePatternMatcher.class);


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private Object vResult = null;
    private final List vCandidates = new ArrayList();


    //------------------------------------------------------------
    //- Constructors


    //------------------------------------------------------------
    //- Accessors

    public Object getResult() {
        return vResult;
    }

    protected List getCandidates() {
        return vCandidates;
    }


    //------------------------------------------------------------
    //- Settors

    public void setResult(Object pResult) {
        vResult = pResult;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    public MultiplePatternMatcher add(Pattern pPattern, Handler pHandler) {
        getCandidates().add(new Candidate(pPattern, pHandler));

        return this;
    }


    public boolean on(String pInput) {
        boolean found = false;  // Pessimistic.

        setResult(null);  // reset

        Iterator iterator = getCandidates().iterator();
        while (!found && iterator.hasNext()) {
            Candidate candidate = (Candidate) iterator.next();
            Matcher matcher = candidate.getPattern().matcher(pInput);
            if (matcher.find()) {
                found = true;
                setResult(candidate.getHandler().handle(matcher));
            }
        }

        return found;
    }


    //------------------------------------------------------------
    //- Class Interface Functions



    //------------------------------------------------------------
    //- Inner Classes

    class Candidate {

        private final Pattern vPattern;
        private final Handler vHandler;


        public Candidate(Pattern pPattern, Handler pHandler) {
            vPattern = pPattern;
            vHandler = pHandler;
        }


        public Handler getHandler() {
            return vHandler;
        }

        public Pattern getPattern() {
            return vPattern;
        }
    }


    public abstract interface Handler {
        public Object handle(Matcher pMatcher);
    }


    //------------------------------------------------------------
    //- Main



}
