/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.util;

import java.util.*;

/**
 * A map of String key to a list of T objects.
 * When a new object is added, it is appended to the existing list.  If no
 * previous entries existed, then a new list is created for it.
 *
 * User: murali
 * Date: Nov 27, 2007
 * Time: 1:00:04 PM
 */
public class NamedListMap<T> {

    //------------------------------------------------------------
    //- Class Variables

    private static final List UNDEFINED = Collections.emptyList();

    //------------------------------------------------------------
    //- Class Functions

    //------------------------------------------------------------
    //- Instance Variables

    private final HashMap<String,List<T>> fUnderlying = new HashMap<String,List<T>>();


    //------------------------------------------------------------
    //- Constructors

    //------------------------------------------------------------
    //- Getters



    //------------------------------------------------------------
    //- Settors



    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    public NamedListMap<T> add(String pKey, T pVal) {
        List<T> list = get(pKey);
        if (list == UNDEFINED) {
            list = new ArrayList<T>();
            fUnderlying.put(pKey, list);
        }
        list.add(pVal);

        return this;
    }


    public List<T> get(String pKey) {
        List<T> candidate = fUnderlying.get(pKey);
        //noinspection unchecked
        return (candidate == null) ? ((List<T>) UNDEFINED) : candidate;
    }


    public boolean contains(String pKey) {
        return fUnderlying.containsKey(pKey);
    }


    public boolean isEmpty() {
        return fUnderlying.isEmpty();
    }



    //------------------------------------------------------------
    //- Class Interface Functions

    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main

}
