/*
   Copyright 2005-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.util;

import java.io.*;
import java.util.regex.*;

import org.apache.log4j.Logger;

/**
 * Read a resource from the jar (classpath actually).
 * 
 * @author  Murali Krishnan
 * @version  $Id: PackageResource.java,v 1.2 2006/06/21 04:09:36 murali Exp $
 */
public abstract class PackageResource {     

    //------------------------------------------------------------  
    //- Class Variables     

    public static final Pattern RE_MULTI_SLASH = Pattern.compile("[/\\\\]+");

    protected static final Logger LOG = Logger.getLogger(PackageResource.class);

    //------------------------------------------------------------  
    //- Class Functions     



    //------------------------------------------------------------  
    //- Instance Variables  

    private String _uri = null;
    private Object _content = null;



    //------------------------------------------------------------  
    //- Constructors        

    public PackageResource(String pUri) {
        String path = RE_MULTI_SLASH.matcher(pUri).replaceAll("/");

        if (!path.startsWith("/")) {
            path = "/" + path;
        }

        _uri = path;
    }


    //------------------------------------------------------------  
    //- Accessors   

    public String getUri() {return _uri;}



    //------------------------------------------------------------  
    //- Settors     



    //------------------------------------------------------------  
    //- Private/Protected Utility Functions 

    protected Object getContent() {
        if (_content == null) {
            _content = readContent();
        }

        return _content;
    }



    //------------------------------------------------------------  
    //- Public Interface Functions  

    public String toString() {
        String result = "Resource[" + getUri() + "]";

        return result;
    }



    //------------------------------------------------------------  
    //- Class Interface Functions   

    protected abstract Object readContent();


    //------------------------------------------------------------  
    //- Inner Classes       



    //------------------------------------------------------------  
    //- Main        



}       
