/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.util;

/**
 * A container for a pair of objects.  Templatized so any types can
 * be used.  The data members can be accessed (by subclasses only)
 * as "first" and "second".
 *
 * This object is immutable (but the objects it contains need not be).
 *
 * User: Murali
 * Date: Jul 28, 2008
 * Time: 8:05:06 AM
 */
public abstract class Pair<T1,T2> {

    //------------------------------------------------------------
    //- Class Variables

    //------------------------------------------------------------
    //- Class Functions

    //------------------------------------------------------------
    //- Instance Variables

    private final T1 fFirst;
    private final T2 fSecond;

    //------------------------------------------------------------
    //- Constructors

    protected Pair(T1 pFirst, T2 pSecond) {
        fFirst = pFirst;
        fSecond = pSecond;
    }

    //------------------------------------------------------------
    //- Accessors

    protected T1 getFirst() {
        return fFirst;
    }

    protected T2 getSecond() {
        return fSecond;
    }

    //------------------------------------------------------------
    //- Settors

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    //------------------------------------------------------------
    //- Public Interface Functions

    public boolean equalsPair(Pair<T1,T2> pOther) {
        return getFirst().equals(pOther.getFirst()) && getSecond().equals(pOther.getSecond());
    }

    //------------------------------------------------------------
    //- Class Interface Functions

    @Override
    public boolean equals(Object pOther) {
        return (pOther instanceof Pair) && equalsPair((Pair<T1,T2>) pOther);
    }


    //------------------------------------------------------------
    //- Inner Classes

    //------------------------------------------------------------
    //- Main

}
