/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.util;

import org.apache.log4j.Logger;

import java.util.Random;

/**
 * A state holder that returns a boolean flag.  The value of the
 * flag is dependent on the defined period.
 *
 * User: Murali
 * Date: Sep 5, 2008
 * Time: 2:40:19 PM
 */
public abstract class PeriodicSelector {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger("com.notuvy.util");

    //------------------------------------------------------------
    //- Class Functions

    //------------------------------------------------------------
    //- Instance Variables

    private int fValue = 0;
    private final int modulus;

    //------------------------------------------------------------
    //- Constructors

    protected PeriodicSelector(int pModulus) {
        modulus = pModulus;
    }

    //------------------------------------------------------------
    //- Accessors

    public int getValue() {
        return fValue;
    }

    public int getModulus() {
        return modulus;
    }

    //------------------------------------------------------------
    //- Settors

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    //------------------------------------------------------------
    //- Public Interface Functions

    public boolean next() {
        return (++fValue % getModulus()) == 0;
    }

    //------------------------------------------------------------
    //- Class Interface Functions

    public String toString() {
        return "PeriodicSelector[every " + getModulus() + " tries]";
    }

    //------------------------------------------------------------
    //- Inner Classes

    public static class Alternate extends PeriodicSelector {
        public Alternate() {
            super (2);
        }
    }

    public static class EveryN extends PeriodicSelector {
        public EveryN(int pModulus) {
            super (pModulus);
        }
    }

    public static class Random extends PeriodicSelector {
        private final java.util.Random fGen = new java.util.Random(System.currentTimeMillis());

        public Random() {
            super (1);  // Irrelevant
        }

        public boolean next() {
            return fGen.nextBoolean();
        }
    }

    //------------------------------------------------------------
    //- Main

    public static void main(String[] args) {
        PeriodicSelector selector = new Alternate();
        for (int i=0; i<10; i++) {
            LOG.info(i + ": " + selector.next());
        }
    }
}
