/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.util;


import java.io.*;
import java.util.HashMap;

/**
 * Read a resource from the jar (classpath actually) and return it as a string.
 * 
 * @author  Murali Krishnan
 * @version  $Id: TextResource.java,v 1.2 2006/06/21 04:09:21 murali Exp $
 */
public class TextResource extends PackageResource {     

    //------------------------------------------------------------  
    //- Class Variables     

    private static final HashMap sCache = new HashMap();



    //------------------------------------------------------------  
    //- Class Functions     

    public static String get(String pUri) {
        String result = (String) sCache.get(pUri);

        if (result == null) {
            TextResource resource = new TextResource(pUri);
            result = resource.toString();
            sCache.put(pUri, result);
        }

        return result;
    }


    //------------------------------------------------------------  
    //- Instance Variables  



    //------------------------------------------------------------  
    //- Constructors        

    protected TextResource(String pUri) {
        super(pUri);
    }


    //------------------------------------------------------------  
    //- Accessors   



    //------------------------------------------------------------  
    //- Settors     



    //------------------------------------------------------------  
    //- Private/Protected Utility Functions 

    protected synchronized Object readContent() {
        String result = "";  // Error case default.

        StringBuffer buffer = new StringBuffer();
        InputStream is = getClass().getResourceAsStream(getUri());
        if (is == null) {
            // Check deprecated location (pre-maven ant builds) for backward compatibility.
            LOG.warn("Found resource in deprecated (/include) location.");
            is = getClass().getResourceAsStream("/include"+getUri());
        }
        try {
            InputStreamReader isr = new InputStreamReader(is);
            BufferedReader br = new BufferedReader(isr);

            for (String line=br.readLine(); line!=null; line=br.readLine()) {
                buffer.append(line).append("\n");
            }

            br.close();
            isr.close();
            is.close();

            result = buffer.toString().trim();
        } catch (Exception ex) {
            // Could be IOException, but would be NullPointerException
            // if uri points to a non-existent resource.
            LOG.error("Could not fetch the text resource [" + getUri() + "].", ex);
        }

        return result;
    }



    //------------------------------------------------------------  
    //- Public Interface Functions  

    public String toString() {
        String result = (String) getContent();

        return result;
    }



    //------------------------------------------------------------  
    //- Class Interface Functions   



    //------------------------------------------------------------  
    //- Inner Classes       



    //------------------------------------------------------------  
    //- Main        



}       
