package com.notuvy.util;

import org.apache.log4j.Logger;

import java.util.Date;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.text.ParseException;

/**
 * A representation of the current time.
 *
 * The default implementation is a trivial encapsulation of the system time.
 * 
 * Its purpose is to be a hook for unit tests to replace with explicit
 * time control.
 *
 * @author murali
 */
public abstract class WallClock {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger("com.notuvy.util");

    public static final WallClock SYSTEM = new SystemClock();

    private static WallClock INSTANCE = SYSTEM;

    public static final String STRING_FORMAT = "yyyy-MM-dd'T'HH:mm:ssZZ";  // E.g. 2010-09-09T14:39:05-0700
    public static final DateFormat FORMATTER = new SimpleDateFormat(STRING_FORMAT);


    //------------------------------------------------------------
    //- Class Functions

    public static long currentTimeMillis() {
        return INSTANCE.getTimeMillis();
    }

    public static Date currentDate() {
        return new Date(currentTimeMillis());
    }

    public static void restoreSystem() {
        INSTANCE = SYSTEM;
    }

    public static Manual createManual() {
        Manual manual = new Manual();
        INSTANCE = manual;
        return manual;
    }

    protected static WallClock getInstance() {
        return INSTANCE;
    }


    //------------------------------------------------------------
    //- Instance Variables


    //------------------------------------------------------------
    //- Constructors


    //------------------------------------------------------------
    //- Accessors


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions


    //------------------------------------------------------------
    //- Class Interface Functions

    protected abstract long getTimeMillis();

    //------------------------------------------------------------
    //- Inner Classes

    static class SystemClock extends WallClock {
        protected long getTimeMillis() {
            return System.currentTimeMillis();
        }
    }

    public static class Manual extends WallClock {

        private long fTimeMillis = 0;

        private Manual() {
        }

        public long getTimeMillis() {
            return fTimeMillis;
        }

        public void setTimeMillis(long pTimeMillis) {
            fTimeMillis = pTimeMillis;
        }

        public void setTime(Date pDate) {
            setTimeMillis(pDate.getTime());
        }

        public void setTime(String pString) {
            try {
                setTime(FORMATTER.parse(pString));
            } catch (ParseException e) {
                LOG.error(String.format("Could not parse [%s] as [%s].", pString, STRING_FORMAT));
            }
        }
    }

    //------------------------------------------------------------
    //- Main


}
