package com.notuvy.xml;

import org.apache.log4j.Logger;

/**
 * The abstract base logic for persisting SimpleJdom.
 *
 * @author murali
 */
public abstract class BaseStorage implements XmlStorable {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger("com.notuvy.xml");


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    protected final SimpleJdom fObject;
    private boolean fOnlySaveIfChanged = false;
    private String fContentSnapshot = "";


    //------------------------------------------------------------
    //- Constructors

    protected BaseStorage(SimpleJdom pObject) {
        fObject = pObject;

        getObject().setStorage(this);
    }


    //------------------------------------------------------------
    //- Gettors

    public SimpleJdom getObject() {
        return fObject;
    }

    public boolean isOnlySaveIfChanged() {
        return fOnlySaveIfChanged;
    }


    //------------------------------------------------------------
    //- Settors

    public void setOnlySaveIfChanged(boolean pOnlySaveIfChanged) {
        fOnlySaveIfChanged = pOnlySaveIfChanged;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    /**
     * There is one situatin where we do a NOOP.  That is if we are checking for
     * changed content, and the contents have not changed.  In that case just return
     * 'success'.  Otherwise return the status of the save operation.
     *
     * @return boolean flag whether the actual save operation should be skipped.
     */
    protected boolean skipSave() {
        return isOnlySaveIfChanged() && getObject().toXmlString().equals(fContentSnapshot);
    }

    protected void takeContentSnapshot() {
        if (isOnlySaveIfChanged()) {
            fContentSnapshot = getObject().toXmlString();
        }
    }


    //------------------------------------------------------------
    //- Public Interface Functions


    //------------------------------------------------------------
    //- Class Interface Functions


    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main

}
