/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.xml;


import org.apache.log4j.NDC;

import java.io.*;

/**
 * Save the XML to the file system.
 *
 * User: murali
 * Date: Dec 30, 2007
 * Time: 7:56:40 AM
 */
public class FileStorage extends BaseStorage {

    //------------------------------------------------------------
    //- Class Variables


    //------------------------------------------------------------
    //- Class Functions

    //------------------------------------------------------------
    //- Instance Variables

    private final File fFileImage;


    //------------------------------------------------------------
    //- Constructors

    public FileStorage(SimpleJdom pObject, File pFileImage) {
        super(pObject);
        fFileImage = pFileImage;
    }

    //------------------------------------------------------------
    //- Accessors

    public File getFileImage() {
        return fFileImage;
    }


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    private boolean saveToStorage() {
        boolean success = false;  // Pessimistic.
        try {
            FileWriter writer = new FileWriter(getFileImage());
            boolean intermediate = getObject().saveTo(writer);
            if (!intermediate) {
                LOG.error("Problem writing to [" + getFileImage().getAbsolutePath() + "]");
            }
            writer.close();

            takeContentSnapshot();

            success = intermediate;
        } catch (IOException ioe) {
            LOG.error("Cannot write to [" + getFileImage().getAbsolutePath() + "]", ioe);
        }
        return success;
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public boolean save() {
        return skipSave() || saveToStorage();
    }

    public boolean read() {
        boolean success = false;  // Pessimistic.
        NDC.push(String.format(" [%s]", getFileImage().getAbsolutePath()));
        try {
            FileReader reader = new FileReader(getFileImage());
            success = getObject().readFrom(reader);
            reader.close();

            if (success) {
                takeContentSnapshot();
            }
        } catch (FileNotFoundException fnfe) {
            LOG.error("Cannot read from [" + getFileImage().getAbsolutePath() + "]", fnfe);
        } catch (IOException ioe) {
            LOG.error("Data read problem.", ioe);
        } finally {
            NDC.pop();
        }
        return success;
    }

    public boolean readIfStoredImageExists() {
        return readIfStoredImageExists(true);
    }

    public boolean readIfStoredImageExists(boolean pVerboseWarn) {
        boolean success = true;  // Vacuously true if no file exists.
        if (getFileImage().exists()) {
            success = read();
        } else if (pVerboseWarn) {
            LOG.warn(String.format("No input file [%s].", getFileImage().getAbsolutePath()));
        }
        return success;
    }

    public boolean readOrCreate() {
        boolean success;
        if (getFileImage().exists()) {
            success = read();
        } else {
            success = saveToStorage();
            LOG.warn("Created: " + getFileImage().getAbsolutePath());
        }
        return success;
    }

    public boolean delete() {
        boolean success = true;  // Optimistic.
        if (getFileImage().exists()) {
            success = getFileImage().delete();
            if (!success) {
                LOG.warn(String.format("Could not delete [%s].", getFileImage().getAbsolutePath()));
            }
        }
        return success;
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    //------------------------------------------------------------
    //- Inner Classes

    //------------------------------------------------------------
    //- Main

}
